const express = require('express');

const populationApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const config = require(`${libs}/config`);

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

populationApp.use(cache('15 day'));

populationApp.get('/year_range', (req, res, next) => {
    req.sql.from('ibge_populacao')
    .field('MIN(ibge_populacao.ano_censo)', 'start_year')
    .field('MAX(ibge_populacao.ano_censo)', 'end_year')
    .where('ibge_populacao.ano_censo <> 2014 AND ibge_populacao.ano_censo <> 2013');
    next();
}, query, response('range'));

populationApp.get('/years', (req, res, next) => {
    req.sql.from('ibge_populacao').
    field('DISTINCT ibge_populacao.ano_censo', 'year')
    .where('ibge_populacao.ano_censo <> 2014 AND ibge_populacao.ano_censo <> 2013');
    next();
}, query, response('years'));

populationApp.get('/city_size', (req, res, next) => {
    req.result = [
        {id: 1, name: "até 5000"},
        {id: 2, name: "5001 - 10000"},
        {id: 3, name: "10001 - 20000"},
        {id: 4, name: "20001 - 50000"},
        {id: 5, name: "50001 - 100000"},
        {id: 6, name: "100001 - 500000"},
        {id: 7, name: "mais que 500000"}
    ];
    next();
}, response('city_size'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: ['nome', 'id'],
    resultField: ['city_name', 'city_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'ibge_populacao'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'ibge_populacao'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
        table: 'ibge_populacao'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'ibge_populacao'
    }
}).addValue({
    name: 'region',
    table: 'regiao',
    tableField: 'nome',
    resultField: 'region_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'regiao_id',
        table: 'ibge_populacao'
    },
    join: {
        primary: 'id',
        foreign: 'regiao_id',
        foreignTable: 'ibge_populacao'
    }
}).addValue({
    name: 'min_year',
    table: 'ibge_populacao',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'ibge_populacao',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'city_size',
    table: 'ibge_populacao',
    tableField: 'porte',
    resultField: 'city_size_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'porte'
    }
});

populationApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
  log.debug(req.sql.toParam());
  log.debug(req.dims);
  req.sql.from('ibge_populacao')
    .field('SUM(ibge_populacao.populacao)', 'total')
    .field('ibge_populacao.ano_censo', 'year')
    .group('ibge_populacao.ano_censo')
    .order('ibge_populacao.ano_censo')

   next();
}, query, id2str.transform(false), response('population'));

module.exports = populationApp;
